#include <stdio.h>
#include <map>
#include <list>
#include <string>
#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include "atcg.h"
#include "functions.h"

using namespace std;

class MultiplexPos;

typedef list<MultiplexPos>PLIST;
typedef map<string, PLIST>PLMAP;

enum CMPBY { RPOSFROM, RPOSTO, JPOSFROM, JPOSTO };

//=====================================================================
class MultiplexPos {
  string m_sRealName;
  string m_sJoinName;
  long m_iRFrom, m_iRTo;
  long m_iJFrom, m_iJTo;
public:
  static CMPBY siCmpMode;
//---------------------------------------------------------------------
  MultiplexPos() {}
//---------------------------------------------------------------------
  MultiplexPos(const char *pcRealName, long iRFrom, long iRTo,
	       const char *pcJoinName, long iJFrom, long iJTo) {
    assign(pcRealName, iRFrom, iRTo, pcJoinName, iJFrom, iJTo);
  }
//---------------------------------------------------------------------
  MultiplexPos(const string &sRealName, long iRFrom, long iRTo,
	       const string &sJoinName, long iJFrom, long iJTo) {
    assign(sRealName, iRFrom, iRTo, sJoinName, iJFrom, iJTo);
  }
//---------------------------------------------------------------------
  void assign(const char *pcRealName, long iRFrom, long iRTo,
	      const char *pcJoinName, long iJFrom, long iJTo) {
    m_sRealName = pcRealName;
    m_sJoinName = pcJoinName;
    m_iRFrom = iRFrom;
    m_iRTo = iRTo;
    m_iJFrom = iJFrom;
    m_iJTo = iJTo;
  }
//---------------------------------------------------------------------
  void assign(const string &sRealName, long iRFrom, long iRTo,
	      const string &sJoinName, long iJFrom, long iJTo) {
    m_sRealName = sRealName;
    m_sJoinName = sJoinName;
    m_iRFrom = iRFrom;
    m_iRTo = iRTo;
    m_iJFrom = iJFrom;
    m_iJTo = iJTo;
  }
//---------------------------------------------------------------------
  long getCmpVal() const {
    switch(siCmpMode) {
    case RPOSTO:
      return m_iRTo;
      break;
    case JPOSTO:
      return m_iJTo;
      break;
    case JPOSFROM:
      return m_iJFrom;
      break;
    case RPOSFROM:
    default:
      return m_iRFrom;
    }
  }
//---------------------------------------------------------------------
  MultiplexPos& operator = (MultiplexPos& mpos) {
    assign(mpos.getRealName(), mpos.getRFrom(), mpos.getRTo(),
	   mpos.getJoinName(), mpos.getJFrom(), mpos.getJTo());
    return *this;
  }
//---------------------------------------------------------------------
  bool operator > (const MultiplexPos& mpos) const {
    return getCmpVal() > mpos.getCmpVal();
  }
//---------------------------------------------------------------------
  bool operator < (const MultiplexPos& mpos) const {
    return getCmpVal() < mpos.getCmpVal();
  }
//---------------------------------------------------------------------
  bool operator >= (const MultiplexPos& mpos) const {
    return getCmpVal() >= mpos.getCmpVal();
  }
//---------------------------------------------------------------------
  bool operator <= (const MultiplexPos& mpos) const {
    return getCmpVal() <= mpos.getCmpVal();
  }
//---------------------------------------------------------------------
  bool operator == (const MultiplexPos& mpos) const {
    return getCmpVal() == mpos.getCmpVal();
  }
//---------------------------------------------------------------------
  bool operator != (const MultiplexPos& mpos) const {
    return getCmpVal() != mpos.getCmpVal();
  }
//---------------------------------------------------------------------
  const string& getRealName() const { return m_sRealName; }
//---------------------------------------------------------------------
  const string& getJoinName() const { return m_sJoinName; }
//---------------------------------------------------------------------
  long getRFrom() const { return m_iRFrom; }
//---------------------------------------------------------------------
  long getRTo() const { return m_iRTo; }
//---------------------------------------------------------------------
  long getJFrom() const { return m_iJFrom; }
//---------------------------------------------------------------------
  long getJTo() const { return m_iJTo; }
//---------------------------------------------------------------------
  void print(FILE *FOUT) {
    fprintf(FOUT, "%s %ld %ld %s %ld %ld\n", getJoinName().c_str(), getJFrom(), getJTo(), getRealName().c_str(), getRFrom(), getRTo());
  }
//---------------------------------------------------------------------
};

//=====================================================================
int loadPosMap(FILE *FIN, PLMAP &plmap) {
  PLMAP::iterator mptr;
  char sLine[MAXLINELENGTH];
  char sRealName[MAXNAMESIZE];
  char sJoinName[MAXNAMESIZE];
  long iRFrom, iRTo, iJFrom, iJTo;
  while(fgets(sLine, MAXLINELENGTH, FIN) != NULL) {
    if(sscanf(sLine, "%s %ld %ld %s %ld %ld", sJoinName, &iJFrom, &iJTo, sRealName, &iRFrom, &iRTo) >= 6) {
      plmap[sJoinName].push_back(*(new MultiplexPos(sRealName, iRFrom, iRTo, sJoinName, iJFrom, iJTo)));
    }
  }
  for(mptr = plmap.begin(); mptr != plmap.end(); mptr++) {
    (*mptr).second.sort();
  }
  return plmap.size();
}
#ifdef DMX2
//=====================================================================
void demultiplexMap(FILE *FIN, FILE *FOUT, PLMAP &plmap) {
  char sLine[MAXLINELENGTH];
  char sTName[MAXNAMESIZE];
  char sRName[MAXNAMESIZE];
  long iTFrom, iTTo, iRFrom, iRTo;
  long iTF, iTT;
  double iScore;
  double dSim, dTranz;
  bool isDirect;
  while(fgets(sLine, MAXLINELENGTH, FIN) != NULL) {
#ifdef DEBUG
    cerr << "DEBUG:" << sLine;
#endif
    if(read_ft(sLine,sRName,iRFrom,iRTo,sTName,iTFrom,iTTo,iScore,dSim,dTranz,isDirect) >= 9){
      if(plmap.find(sTName) != plmap.end()) {
	PLIST::iterator lptr;
	for(lptr = plmap[sTName].begin(); lptr != plmap[sTName].end(); lptr++) {
// [  |.........|  ]
	  if(iTFrom >= (*lptr).getJFrom() && iTTo <= (*lptr).getJTo()) {
#ifdef DEBUG
	    cerr << "DEBUG: Case 1\n";
#endif
	    iTF = iTFrom - (*lptr).getJFrom() + (*lptr).getRFrom();
	    iTT = iTTo - (*lptr).getJFrom() + (*lptr).getRFrom();
	    print_ft(FOUT,sRName,iRFrom,iRTo,(*lptr).getRealName().c_str(),iTF,iTT,iScore,dSim,dTranz,isDirect);
	    break;
	  }
// [  |.......]....|
	  if(iTFrom >= (*lptr).getJFrom() && iTTo > (*lptr).getJTo() && iTFrom < (*lptr).getJTo()) {
#ifdef DEBUG
	    cerr << "DEBUG: Case 2\n";
#endif
	    iTF = iTFrom - (*lptr).getJFrom() + (*lptr).getRFrom();
	    iTT = (*lptr).getRTo();
	    print_ft(FOUT,sRName,iRFrom,iRTo,(*lptr).getRealName().c_str(),iTF,iTT,iScore,dSim,dTranz,isDirect);
	    iTFrom = (*lptr).getJTo() + 2;
	    lptr--;
	    lptr--;
	  }
// |......[.....|  ]
	  if(iTFrom < (*lptr).getJFrom() && iTTo <= (*lptr).getJTo() && iTTo > (*lptr).getJFrom()) {
#ifdef DEBUG
	    cerr << "DEBUG: Case 3\n";
#endif
	    iTF = 1;
	    iTT = iTTo - (*lptr).getJFrom() + (*lptr).getRFrom();
	    print_ft(FOUT,sRName,iRFrom,iRTo,(*lptr).getRealName().c_str(),iTF,iTT,iScore,dSim,dTranz,isDirect);
	    iTTo = (*lptr).getJFrom() - 2;
	    lptr--;
	    lptr--;
	  }
	}
      }
      else {
	print_ft(FOUT, sRName,iRFrom,iRTo,sTName,iTFrom,iTTo,iScore,dSim,dTranz,isDirect);
      }
    }
  }
}
#else
//=====================================================================
void demultiplexMap(FILE *FIN, FILE *FOUT, PLMAP &plmap) {
  char sLine[MAXLINELENGTH];
  char sTName[MAXNAMESIZE];
  char sRName[MAXNAMESIZE];
  long iTFrom, iTTo, iRFrom, iRTo;
  long iTF, iTT;
  double iScore;
  double dSim, dTranz;
  bool isDirect;
  while(fgets(sLine, MAXLINELENGTH, FIN) != NULL) {
    if(read_ft(sLine,sTName,iTFrom,iTTo,sRName,iRFrom,iRTo,iScore,dSim,dTranz,isDirect) >= 9){
#ifdef DEBUG
      cerr << "DEBUG:" << sLine;
#endif
      if(plmap.find(sTName) != plmap.end()) {
	PLIST::iterator lptr;
	for(lptr = plmap[sTName].begin(); lptr != plmap[sTName].end(); lptr++) {
// [  |.........|  ]
	  if(iTFrom >= (*lptr).getJFrom() && iTTo <= (*lptr).getJTo()) {
#ifdef DEBUG
	    cerr << "DEBUG: Case 1\n";
#endif
	    iTF = iTFrom - (*lptr).getJFrom() + (*lptr).getRFrom();
	    iTT = iTTo - (*lptr).getJFrom() + (*lptr).getRFrom();
	    print_ft(FOUT,(*lptr).getRealName().c_str(),iTF,iTT,sRName,iRFrom,iRTo,iScore,dSim,dTranz,isDirect);
	    break;
	  }
// [  |.......]....|
	  if(iTFrom >= (*lptr).getJFrom() && iTTo > (*lptr).getJTo() && iTFrom < (*lptr).getJTo()) {
#ifdef DEBUG
	    cerr << "DEBUG: Case 2\n";
#endif
	    iTF = iTFrom - (*lptr).getJFrom() + (*lptr).getRFrom();
	    iTT = (*lptr).getRTo();
	    print_ft(FOUT,(*lptr).getRealName().c_str(),iTF,iTT,sRName,iRFrom,iRTo,iScore,dSim,dTranz,isDirect);
	    iTFrom = (*lptr).getJTo() + 2;
	    lptr--;
	    lptr--;
	  }
// |......[.....|  ]
	  if(iTFrom < (*lptr).getJFrom() && iTTo <= (*lptr).getJTo() && iTTo > (*lptr).getJFrom()) {
#ifdef DEBUG
	    cerr << "DEBUG: Case 3\n";
#endif
	    iTF = 1;
	    iTT = iTTo - (*lptr).getJFrom() + (*lptr).getRFrom();
	    print_ft(FOUT,(*lptr).getRealName().c_str(),iTF,iTT,sRName,iRFrom,iRTo,iScore,dSim,dTranz,isDirect);
	    iTTo = (*lptr).getJFrom() - 2;
	    lptr--;
	    lptr--;
	  }
	}
      }
      else {
	print_ft(FOUT, sTName,iTFrom,iTTo,sRName,iRFrom,iRTo,iScore,dSim,dTranz,isDirect);
      }
    }
  }
}
#endif //DMX2
//=====================================================================
void usage(char *exec) {
  fprintf(stderr, "\n====================================================\n");
  fprintf(stderr, " Usage: %s [infile] conv_map [out_file]\n", exec);
#ifdef DMX2
  fprintf(stderr, "Applies to columns 4,5,6 of the map\n");
#else
  fprintf(stderr, "Applies to columns 1,2,3 of the map\n");
#endif
  fprintf(stderr, "\n====================================================\n");
}

CMPBY MultiplexPos::siCmpMode = JPOSFROM;

//=====================================================================
int main(int argc, char **argv) {
  FILE *FIN, *FMAP, *FOUT;
  char **args = argv;

  if((args = assigninput(FIN, args)) ==  NULL) {
    usage(argv[0]);
    exit(1);
  }
  if((FMAP = openifilename(args[1])) == 0) {
    usage(argv[0]);
    exit(20);
  }
  args++;
  if((args = assignoutput(FOUT, args)) ==  NULL) {
    usage(argv[0]);
    exit(30);
  }
//---------------------------------------------------------------------
  PLMAP plmap;
  try {
    loadPosMap(FMAP, plmap);
    demultiplexMap(FIN, FOUT, plmap);
  }
  catch(exception exc) {
    cerr << argv[0] << ": Cought exception: " << exc.what() << "\n";
  }
  catch(...) {
    cerr << argv[0] << ": Bad exception\n";
  }
//---------------------------------------------------------------------
  FIN == stdin || fclose(FIN);
  fclose(FMAP);
  fflush(FOUT);
  FOUT == stdout || fclose(FOUT);
  return 0;
}
