/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fri Dec  6 15:19:06 PST 2002
    copyright            : (C) 2002 by Oleksiy Kohany
    email                : kohany@girinst.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#define IOSTREAM

#include <iostream>
#include <stdlib.h>
#include <string.h>
#include "cseqstat.h"
#include "functions.h"
#include "constants.h"
using namespace bio;

void usage(char* exec) {
   if(!exec) {
      return;
   }
   if(strrchr(exec, '/')) {
      exec = strrchr(exec, '/') + 1;
   }
   cerr << "NAME\n\t" << exec;
#ifdef VERSION
   cerr << " V" << VERSION;
#endif
   cerr << " - calculates length and gueses type of genomic sequence (aminoacid \n\tor nucleotide)\n";
   cerr << "\nSYNOPSIS\n\t" << exec << " [infile(-)] [outfile] [-g]\n";
   cerr << "\nDESCRIPTION\n\tCalculates length and densities of ATCG bases in genomic sequence\n";
   cerr << "\tIf total density of ATCG is higher than 0.75, assumes that the sequence\n";
   cerr << "\tis nucleic acid sequence, otherwise - aminoacid. In addition it reports\n";
   cerr << "\tsize and CG content (only for nucleotide sequences) for each of processed\n";
   cerr << "\tsequences.\n";
   cerr << "\nOPTIONS\n";
   cerr << "\t-g - if specified, a line with summary for all processed sequences\n";
   cerr << "\twill be added to the tail of the report.\n";
   cerr << "\t-n - include 'N' and 'X' symbols in calculating length.\n";
#ifdef PACKAGE_BUGREPORT
   cerr << "\nBUG REPORT\n\t" << PACKAGE_BUGREPORT << "\n";
#endif
#ifdef COPYRIGHT
   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
#ifdef BUILD_NUMBER
   cerr << "\nBuild " << BUILD_NUMBER << "\n";
#endif
}

int main(int argc, char *argv[])
{
  istream *is = &cin;
  ostream *os = &cout;
  bool showGlobal = 0;
  char **args = argv;
  ARGMAP amap;
  ::getflags(argv, amap, 4, "--help", "--version", "-g", "-n");
  if(amap.find("-g") != amap.end()) {
     showGlobal = 1;
  }
  if(amap.find("-n") != amap.end()) {
     CSeqStat::sbWithN = 1;
  }
  if((args = assigninput(is, argv)) == 0) {
    usage(argv[0]);
    exit(10);
  }
  if((args = assignoutput(os, args)) == 0) {
    usage(argv[0]);
    exit(20);
  }
//=========================================================
  CSeqStat sst;
  CSeqStat SST;
  SST.setName("Global");
  try {
    while(sst.read(*is) >= 0) {
      sst.print(os);
      if(showGlobal) {
         SST += sst;
      }
    }
    if(showGlobal) {
       SST.print(os);
    }
  }
  catch(out_of_range& exc) {
    cerr << argv[0] << ": " << exc.what() << "\n";
    exit(40);
  }
//=========================================================
  if(is != &cin) {
     delete is;
  }
  if(os != &cout) {
     delete os;
  }
  return 0;
}
